"""
Created on 2012-09-05

@author: skardan
"""
from skgaze.EMDAT.Participant import Participant
#def explore_validation_threshold_segments(datadir, prune_length = None, user_list = xrange(7,38),
#                  auto_partition_low_quality_segments = False):
#    
#    participants = []
#    pid = 55
#    seglen = 0
#    segs = 0
#    for rec in user_list:
#        print "pid:", pid
#        if rec<10:
#            allfile = datadir+'/Rec 0'+str(rec)+'-All-Data.tsv'
#            fixfile = datadir+'/Rec 0'+str(rec)+'-Fixation-Data.tsv'
#            evefile = datadir+'/Rec 0'+str(rec)+'-Event-Data.tsv'
#        else:
#            allfile = datadir+'/Rec '+str(rec)+'-All-Data.tsv'
#            fixfile = datadir+'/Rec '+str(rec)+'-Fixation-Data.tsv'
#            evefile = datadir+'/Rec '+str(rec)+'-Event-Data.tsv'
#        print allfile
#        import os.path
#        if os.path.exists(allfile):
#            p = Participant(pid, evefile, allfile, fixfile, aoifiles=None, prune_length = prune_length,
#                                require_valid_segs = False,
#                                auto_partition_low_quality_segments = auto_partition_low_quality_segments)
#            tvalidity = []
#            
#            for seg in p.segments:
#                seglen += seg.completion_time
#            segs += len(p.segments)
#
##            for tresh in range(1,100,1):##proportion
##                invc = 0
##                invsegs=[]
##                for seg in segments:
##                    if seg.calc_validity1(tresh/100.0) == False:
##                        invc +=1
##            for tresh in range(1,21,1):##prop-gap
##                invc = 0
##                invsegs=[]
##                for seg in segments:
##                    if seg.calc_validity2(tresh*seg.completion_time/100.0) == False:
##                        invc +=1                        
##            for tresh in range(1,20000,100):##time-gap
##                invc = 0
##                invsegs=[]
##                for seg in segments:
##                    if seg.calc_validity2(tresh) == False:
##                        invc +=1
#
#            for tresh in range(1,102,1):##proportion Fixation
#                invc = 0
#                invsegs=[]
#                for seg in p.segments:
#                    if seg.calc_validity3(tresh/100.0) == False:
#                        invc +=1                        
#                        invsegs.append(seg.segid)
#                        
#                if len(invsegs)>0:
#                    print "seg:",invsegs 
#                        
#                tvalidity.append((tresh, invc))
#            participants.append( (pid,tvalidity, len(p.segments) ) )
#            print ( (tvalidity, len(p.segments)) )
#           
#        pid += 1
#        print "average seg len",seglen/float(segs)
#    return participants

def explore_validation_proportion_threshold_segments(participant_list, include_restored_samples = True, prune_length = None, 
                                          auto_partition_low_quality_segments = False):
    """Explores different threshiold values for the proportion of valid samples method in terms of Segments for all Participants in the list
    """
    
    seglen = 0
    segs = 0
    participants = []
    for p in participant_list:
        print("pid:", p.pid)
        if p.require_valid_segments == True:
            raise Exception("explore_validation_threshold_segments should be called with a list of Participants with require_valid_segments = False")
            
        
        tvalidity = []
        
        for seg in p.segments:
            seglen += seg.completion_time
        segs += len(p.segments)
        if include_restored_samples == False:
            for tresh in range(1,102,1):    ##proportion
                invc = 0
                invsegs=[]
                for seg in p.segments:
                    if seg.calc_validity1(tresh/100.0) == False:
                        invc +=1

        else:
            for tresh in range(1,102,1):    ##proportion with restored samples from Fixations
                invc = 0
                invsegs=[]
                for seg in p.segments:
                    if seg.calc_validity3(tresh/100.0) == False:
                        invc +=1                        
                        invsegs.append(seg.segid)
                        
                if len(invsegs)>0:
                    print("seg:",invsegs)
                        
                tvalidity.append((tresh, invc))
        participants.append( (p.pid,tvalidity, len(p.segments) ) )
        print ( (tvalidity, len(p.segments)) )
       
    print("average seg len",seglen/float(segs))
    return participants

def explore_validation_time_gap_threshold_segments(participant_list, time_gap_list = [100, 200, 300, 400, 500, 1000, 2000], prune_length = None, 
                                          auto_partition_low_quality_segments = False):
    """Explores different threshiold values for the invalid time gaps in the Segments for all Participants in the list
    """
    
    seglen = 0
    segs = 0
    participants = []
    for p in participant_list:
        print("pid:", p.pid)
        if p.require_valid_segments == True:
            raise Exception("explore_validation_threshold_segments should be called with a list of Participants with require_valid_segments = False")
            
        
        tvalidity = []
        
        for seg in p.segments:
            seglen += seg.completion_time
        segs += len(p.segments)

                       
        for tresh in time_gap_list:     ##time-gap
            invc = 0
            invsegs=[]
            for seg in p.segments:
                if seg.calc_validity2(tresh) == False:
                    invc +=1

            if len(invsegs)>0:
                print("seg:",invsegs)
                    
            tvalidity.append((tresh, invc))
        participants.append( (p.pid,tvalidity, len(p.segments) ) )
        print ( (tvalidity, len(p.segments)) )
       
    print("average seg len",seglen/float(segs))
    return participants


def explore_validation_proportion_threshold_participants(participant_list, include_restored_samples =True, prune_length = None,
                   auto_partition_low_quality_segments = False):
    """Explores different threshiold values for the proportion of valid samples method for evaluating the validity of each Particiapnt in the list
    """
    participants = []
    seglen = 0
    segs = 0
    for p in participant_list:
        print("pid:", p.pid)
        tvalidity = []
            
        for seg in p.segments:
            seglen += seg.completion_time
        segs += len(p.segments)
        
        if include_restored_samples == False:
            for tresh in range(1,102,1):    ##proportion 
                invc = 0
                if p.is_valid(method=1, threshold = tresh/100.0) == False:
                    invc +=1                        
                tvalidity.append((tresh, invc))
        else:
            for tresh in range(1,102,1):    ##proportion Fixation
                invc = 0
                if p.is_valid(method=3, threshold = tresh/100.0) == False:
                    invc +=1                        
                tvalidity.append((tresh, invc))

        participants.append( (p.pid,tvalidity, len(p.segments) ) )
        print ( (tvalidity, len(p.segments)) )
                 
    return participants

def output_Validity_info_Participants(user_list, include_restored_samples, auto_partition_low_quality_segments_flag):
    """ Outputs the Validuty info for Participants for different Threshold values
    """
    pv = explore_validation_proportion_threshold_participants(participant_list=user_list, include_restored_samples = include_restored_samples, prune_length = None, 
                        auto_partition_low_quality_segments = auto_partition_low_quality_segments_flag)
    
    for rate in range(1,102,1): ##porportion
        usr=[]
        totalseg = 0
        inv_user = 0
        for p in pv:
            pid, i,t = p
            totalseg += t
            _,invc = i[rate-1] 
            if invc > 0 :
                inv_user+=1
                usr.append(invc)
            else:
                usr.append(0)
            
    #    print rate*100,":",inv_seg,"/",totalseg,"user:",inv_user,":",usr
        print(rate,": users with invalid Segment:",inv_user,":",usr)
    print()
    print("Total Segments:",totalseg)
##########


def output_Validity_info_Segments(user_list, auto_partition_low_quality_segments_flag, validity_method, threshold_gaps_list = [], output_file= None):
    """ Outputs the Validuty info for Segments over all Participants for different Threshold values
    """
    if output_file:
        print("writing results to: " + output_file)
        ofile = open(output_file,"w")
        s="Threshold,inv_seg,totalseg,# of Participants with invalid Segment,# of invalid Segements for each Participant"+"\n"
        ofile.write(s)
        print()
        if validity_method == 1|validity_method == 3:   ##porportion
            pv = explore_validation_proportion_threshold_segments(participant_list=user_list, prune_length = None,
                               auto_partition_low_quality_segments = auto_partition_low_quality_segments_flag)
            for rate in range(1,102,1): 
                usr=[]
                totalseg = 0
                inv_seg = 0
                inv_user = 0
                for p in pv:
                    pid, i,t = p
                    totalseg += t
                    _,invc = i[rate-1] 
                    inv_seg += invc
                    if invc > 0 :
                        inv_user+=1
                        usr.append(invc)
                    else:
                        usr.append(0)
                print("writin:"+str(rate))
                s= str(rate),","+str(inv_seg),","+str(totalseg)+","+str(inv_user)+","+str(usr)+"\n"
                ofile.write(s)
    
        elif validity_method == 2:  ##time gap\
            pv = explore_validation_time_gap_threshold_segments(participant_list = user_list, time_gap_list = threshold_gaps_list, prune_length = None, 
                                                                auto_partition_low_quality_segments = auto_partition_low_quality_segments_flag)
            for gap_index in range(len(threshold_gaps_list)): 
                usr=[]
                totalseg = 0
                inv_seg = 0
                inv_user = 0
                for p in pv:
                    pid, i,t = p
                    totalseg += t
                    _,invc = i[gap_index]    #rate-1 
                    inv_seg += invc
                    if invc > 0 :
                        inv_user+=1
                        usr.append(invc)
                    else:
                        usr.append(0)
                    
                print("writin:"+str(threshold_gaps_list[gap_index]))
                s = str(threshold_gaps_list[gap_index])+","+str(inv_seg)+","+str(totalseg)+","+str(inv_user)+","+str(usr)+"\n"
                ofile.write(s)
        print("Finished writing to file")
        ofile.close()
    else:    
        print
        if validity_method == 1|validity_method == 3:   ##porportion
            pv = explore_validation_proportion_threshold_segments(participant_list=user_list, prune_length = None,
                               auto_partition_low_quality_segments = auto_partition_low_quality_segments_flag)
            for rate in range(1,102,1): 
                usr=[]
                totalseg = 0
                inv_seg = 0
                inv_user = 0
                for p in pv:
                    pid, i,t = p
                    totalseg += t
                    _,invc = i[rate-1] 
                    inv_seg += invc
                    if invc > 0 :
                        inv_user+=1
                        usr.append(invc)
                    else:
                        usr.append(0)
                    
                print(rate,":",inv_seg,"/",totalseg,", users with invalid Segment::",inv_user,":",usr)
    
        elif validity_method == 2:  ##time gap\
            pv = explore_validation_time_gap_threshold_segments(participant_list = user_list, time_gap_list = threshold_gaps_list, prune_length = None, 
                                                                auto_partition_low_quality_segments = auto_partition_low_quality_segments_flag)
            for gap_index in range(len(threshold_gaps_list)): 
                usr=[]
                totalseg = 0
                inv_seg = 0
                inv_user = 0
                for p in pv:
                    pid, i,t = p
                    totalseg += t
                    _,invc = i[gap_index]    #rate-1 
                    inv_seg += invc
                    if invc > 0 :
                        inv_user+=1
                        usr.append(invc)
                    else:
                        usr.append(0)
                    
                print(threshold_gaps_list[gap_index],":",inv_seg,"/",totalseg,", users with invalid Segment:",inv_user,":",usr)        
##################



########### PRINT scence validity
#print
#actionids = [1    ,2    ,3    ,4    ,9    ,10    ,11]
#print '\t',repr(actionids).strip('[').rstrip(']').replace(',','\t')
#for p in ps:
#    validscs = map(lambda x:x.scid, filter(lambda x:x.is_valid,p.scenes))
#    output = map(lambda x:int(x in validscs),actionids)
#    print p.id,'\t',(repr(output)).strip('[').rstrip(']').replace(',','\t')
#print
#for p in ps:
#    existingscs = filter(lambda x:x.scid in actionids,p.scenes)
#    exscdic = {}
#    for sc in existingscs:
#        exscdic[sc.scid] = sc
#        
#    def retnumberofsegs(dic,acid):
#        if acid in dic.keys():
#            return dic[acid].features['numsegments']
#        else:
#            return 0
#    output = map(lambda x:retnumberofsegs(exscdic,x),actionids)
#    print p.id,'\t',(repr(output)).strip('[').rstrip(']').replace(',','\t')    


###### PRINT invalid segements 
#for p in ps:
#    ##p.print_()
#    print p.id
#    print p.invalid_segments()
#    print p.valid_segments() 
    
def output_percent_discarded(participant_list, output_file= None):
    """percent of data Discarded due to validity
    """
    if output_file:
        print("writing results to: "+output_file)
        ofile = open(output_file,"w")
        s="pid,is_valid,valid_duration,total_duration,valid_portion"+"\n"
        ofile.write(s)
        for p in participant_list:
            if p.require_valid_segments == True:
                raise Exception("output_percent_discarded should be called with a list of Participants with require_valid_segments = False")

            vlength_sum = sum(map (lambda y:y.length,filter(lambda x:x.is_valid,p.segments)))
            length_sum = sum(map(lambda x:x.length,p.segments))
            s=str(p.pid)+","+str(p.is_valid())+","+str(vlength_sum)+","+str(length_sum)+","+str(vlength_sum*1.0/length_sum)+"\n"
            ofile.write(s)
        ofile.close()
    else:
        print("pid,is_valid,valid_duration,total_duration,valid_portion")
        for p in participant_list:
            vlength_sum = sum(map (lambda y:y.length,filter(lambda x:x.is_valid,p.segments)))
            length_sum = sum(map(lambda x:x.length,p.segments))
            print(p.pid,p.is_valid(),vlength_sum,length_sum,vlength_sum*1.0/length_sum)
        
    ###### 



"""
UBC Eye Movement Data Analysis Toolkit

$$$Daria: contains auxiliary functions for math calculations performed within EMDAT

Author: Nicholas FitzGerald - nicholas.fitzgerald@gmail.com

Basic geometrical helper functions
"""
import os, sys, math, random


def euclidean_distance(point1, point2):
    (x1, y1) = point1
    (x2, y2) = point2

    x1 = float(x1)
    y1 = float(y1)
    x2 = float(x2)
    y2 = float(y2)

    return math.sqrt((x1-x2)**2 + (y1-y2)**2)


def vector_difference(point1, point2):
    (x1, y1) = point1
    (x2, y2) = point2

    distance = euclidean_distance(point1, point2)

    if distance == 0:
        return 0,0
    
    (dx, dy) = (x2-x1, y2-y1)

    signx = dx > 0
    signy = dy > 0

    if dx ==0 and dy!=0:
        return distance, 0
    elif dx!=0 and dy==0:
        theta = math.pi/2
    else:
        theta = math.atan(float(abs(dy))/abs(dx))

    if signx and signy:
        return distance, theta
    elif signx and not signy:
        return distance, -theta
    elif not signx and signy:
        return distance, math.pi - theta
    elif not signx and not signy:
        return distance, theta - math.pi

    raise Exception()

def vector2coords(mag, angle):
    return (mag*math.cos(angle), mag*math.sin(angle))
   
def random_angle():
    return random.uniform(-math.pi, math.pi)

def opp_direction(direction):
    return direction-math.pi

def random_vector_coords(mag):
    return vector2coords(mag,random_angle())

def vector_add(vec1, vec2):
    (x1, y1) = vec1
    (x2, y2) = vec2
    return (x1+x2, y1+y2)

def add_random_scatter(target, mag):
    return vector_add(target, random_vector_coords(mag))

def rads2degrees(angle):
    return angle*180/math.pi



import skgaze.RQA.CustomPythonModules.DynamicalSystemsModule as RecurrenceFunctions;
import skgaze.RQA.CustomPythonModules.SpatioTemporalEyeTrackingModule as ReoccurrenceFunctions;
import os
import sys
import math

class RQA:
    
    def __init__(self,fixations):
        self.fixations = fixations
        self.TimeSeriesData = []
        self.loadFileInfo()


    def loadTimeSeriesData(self):
        for i in range( 0, self.numFixations, 1 ):
            self.TimeSeriesData.append((self.fixationPositionsX[i], self.fixationPositionsY[i]))
            self.TimeSeriesDimensions = len( self.TimeSeriesData[0])

    def loadFileInfo(self):

        self.numFixations = len(self.fixations)
        self.fixationPositionsX = []
        self.fixationPositionsY = []
        self.fixationDurations = []
        for fixation in self.fixations:
            self.fixationPositionsX.append( float(fixation[0]))
            self.fixationPositionsY.append( float(fixation[1]))
            self.fixationDurations.append( float(fixation[2]))

            #rof
        self.fixationDurationsScaled = []
        maxFixationDuration = max( self.fixationDurations )
        for i in range( 0, self.numFixations, 1 ):
            scaledFixation = ( ( float( self.fixationDurations[i] ) / float( maxFixationDuration ) ))

            self.fixationDurationsScaled.append( scaledFixation)
        
        return ( None )
        #fed -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------



        ## FUNCTION FOR CALCULATING AND PLOTTING THE RECURRENCE DATA AND STATS
        # --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    def CalculateRecurrenceFunction( self, timeDelayValue, numTimeDelaySample, phaseSpaceClusteringThreshold ):

        result = {}
        self.TimeDelayValue = timeDelayValue
        self.loadTimeSeriesData()
        

        self.minEmbeddingSequenceLength = int(math.ceil(((2.0 * self.TimeSeriesDimensions) + 1.0 ) / float(self.TimeSeriesDimensions)))
        self.NumTimeDelaySamples = numTimeDelaySample
        self.PhaseSpaceClusteringThreshold = phaseSpaceClusteringThreshold

        self.FixationsXYPhaseSpaceData = RecurrenceFunctions.TimeDelayEmbedding(timeSeriesObservations=self.TimeSeriesData,delayStep=self.TimeDelayValue,delaySamples=self.NumTimeDelaySamples )
        self.RecurrenceMatrixData = RecurrenceFunctions.CreateRecurrenceMatrix(phaseSpaceData=self.FixationsXYPhaseSpaceData,clusteringDistanceThreshold=self.PhaseSpaceClusteringThreshold )

        self.Recurrence = RecurrenceFunctions.getRecurrence(self.RecurrenceMatrixData,self.NumTimeDelaySamples )
        
        self.RecurrenceRate = RecurrenceFunctions.getRecurrenceRate( self.RecurrenceMatrixData, self.NumTimeDelaySamples )
        ( self.RecurrenceMeanX, self.RecurrenceMeanY ) = RecurrenceFunctions.getRecurrenceMean( self.RecurrenceMatrixData, self.NumTimeDelaySamples )
        ( self.RecurrenceStandardDeviationX, self.RecurrenceStandardDeviationY ) = RecurrenceFunctions.getRecurrenceStandardDeviation( self.RecurrenceMatrixData, self.NumTimeDelaySamples )
    
        result['recurrence'] = self.Recurrence
        result['recurrenceRate'] = self.RecurrenceRate
        result['recurrenceMeanX'] = self.RecurrenceMeanX
        result['recurrenceMeanY'] = self.RecurrenceMeanY
        result['recurrenceStandartDeviationX'] = self.RecurrenceStandardDeviationX
        result['recurrenceStandartDeviationY'] = self.RecurrenceStandardDeviationY

            # valueNames= [ r'\rho', r'\bar{\rho}', r'\mu_{x}', r'\mu_{y}', r'\sigma_{x}', r'\sigma_{y}' ],
        return result
        #fed -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------



        ## FUNCTION FOR CALCULATING AND PLOTTING THE REOCCURRENCE DATA AND STATS
        # --------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
    def CalculateReoccurrenceFunction( self, fixationSpaceDistanceRadius):

        if not self.TimeSeriesData:
            self.loadTimeSeriesData()

        result = {}
        self.FixationSpaceDistanceRadius = fixationSpaceDistanceRadius
        self.ReoccurrenceMatrixData = ReoccurrenceFunctions.CreateReoccurrenceMatrix( positionData=self.TimeSeriesData,clusteringDistanceThreshold=self.FixationSpaceDistanceRadius )
        self.Reoccurrence = ReoccurrenceFunctions.getReoccurrence( self.ReoccurrenceMatrixData )
        self.ReoccurrenceRate = ReoccurrenceFunctions.getReoccurrenceRate( self.ReoccurrenceMatrixData )
        self.ReoccurrenceDeterminism = ReoccurrenceFunctions.getDeterminism( self.ReoccurrenceMatrixData )
        self.ReoccurrenceLaminarity = ReoccurrenceFunctions.getLaminarity( self.ReoccurrenceMatrixData )
        self.ReoccurrenceCORM = ReoccurrenceFunctions.getCORM( self.ReoccurrenceMatrixData )

        result['reoccurence'] = self.Reoccurrence
        result['reoccurenceRate'] = self.ReoccurrenceRate
        result['reoccurenceDeterminism'] = self.ReoccurrenceDeterminism
        result['reoccurenceLaminarity'] = self.ReoccurrenceLaminarity
        result['reoccurenceCORM'] = self.ReoccurrenceCORM
        
        return result


def get_longest_common_subsequence(s1, s2):
    # Source: https://rosettacode.org/wiki/Longest_common_subsequence#Dynamic_Programming_8

    lengths = [[0 for j in range(len(s2) + 1)] for i in range(len(s1) + 1)]
    # row 0 and column 0 are initialized to 0 already
    for i, x in enumerate(s1):
        for j, y in enumerate(s2):
            if x == y:
                lengths[i+1][j+1] = lengths[i][j] + 1
            else:
                lengths[i+1][j+1] = max(lengths[i+1][j], lengths[i][j+1])
    # read the substring out from the matrix
    result = ""
    x, y = len(s1), len(s2)
    while x != 0 and y != 0:
        if lengths[x][y] == lengths[x-1][y]:
            x -= 1
        elif lengths[x][y] == lengths[x][y-1]:
            y -= 1
        else:
            assert s1[x - 1] == s2[y - 1]
            result = s1[x - 1] + result
            x -= 1
            y -= 1
    return result



"""
UBC Eye Movement Data Analysis Toolkit
Recording class

Author: Nicholas FitzGerald - nicholas.fitzgerald@gmail.com
Modified by: Samad Kardan to a general class independent of the study

Class to hold all the data from one recording (i.e, one complete experiment session)
for one participant

"""


from skgaze.EMDAT.data_structures import *
from skgaze.EMDAT.Scene import *
#from Dynamic_Segment import *
from skgaze.EMDAT.AOI import *
from skgaze.EMDAT.utils import *
import skgaze.EMDAT.params as params
from warnings import warn


class Recording():
    def __init__(self, all_file, fixation_file, media_offset = (0,0)):
        """
        Args:
            all_file: a string containing the filename of the 'All-Data.tsv' 
                file output by the Tobii software.
                
            fixation_file: a string containing the filename of the 
                'Fixation-Data.tsv' file output by the Tobii software.
                
            media_offset: the coordinates of the top left corner of the window
                showing the interface under study. (0,0) if the interface was
                in full screen (default value)
            
        Yields:
            a Recording object
        """
        self.all_data = read_all_data(all_file)
        if len(self.all_data)==0:
            raise Exception("The file '"+all_file+"' has no samples!")
        self.fix_data = read_fixation_data(fixation_file, 
                                           media_offset = params.MEDIA_OFFSET)
        if len(list(self.fix_data))==0:
            raise Exception("The file '"+fixation_file+"' has no fixations!")

    def process_rec(self, segfile = None, scenelist = None,  aoifile = None, 
                    aoilist = None , prune_length = None, require_valid_segs = True, 
                    auto_partition_low_quality_segments = False):
        """Processes the data for one recording (i.e, one complete experiment session)
        
                
        Args:
            segfile: If not None, a string containing the name of the segfile 
                with segment definitions in following format:
                Scene_ID<tab>Segment_ID<tab>start time<tab>end time<newline>
                e.g.:
                s1    seg1    0    5988013
                With one segment definition per line
            scenelist: If not None, a list of Scene objects
            *Note: At least one of segfile and scenelist should be not None
                
            aoifile: If not None, a string containing the name of the aoifile 
                with definitions of the "AOI"s.
            aoilist: If not None, a list of "AOI"s.
            *Note:  if aoifile is not None, aoilist will be ignored
                    if both aoifile and aoilist are none AOIs are ignored
             
            prune_length: If not None, an integer that specifies the time 
                interval (in ms) from the beginning of each Segment in which
                samples are considered in calculations.  This can be used if, 
                for example, you only wish to consider data in the first 
                1000 ms of each Segment. In this case (prune_length = 1000),
                all data beyond the first 1000ms of the start of the "Segment"s
                will be disregarded.
            
            require_valid_segs: a boolean determining whether invalid "Segment"s
                will be ignored when calculating the features or not. default = True 
            
            auto_partition_low_quality_segments: a boolean flag determining whether
                EMDAT should automatically split the "Segment"s which have low sample quality
                into two new sub "Segment"s discarding the largest invalid sample gap in 
                the "Segment". default = False
        Returns:
            a list of Scene objects for this Recording
            a list of Segment objects for this recording. This is an aggregated list
            of the "Segment"s of all "Scene"s in the Recording 
        """        
      
        if segfile != None:
            scenelist = read_segs(segfile)
            print("Done reading the segments!")
        elif scenelist == None:
            print("Error in scene file")
        
        if aoifile!= None:
            aoilist = read_aois_Tobii(aoifile)
            print("Done reading the AOIs!")
        elif aoilist == None:
            aoilist = []
            print("No AOIs defined!")

#        all_ind = 0
#        fix_ind = 0
        scenes = []

        for scid,sc in scenelist.items():
            print("Preparing scene:"+str(scid))
            if params.DEBUG:
                print("len(all_data)",len(self.all_data))
            
            try:
                newSc = Scene(scid, sc,self.all_data,self.fix_data, aoilist=aoilist, 
                              prune_length=prune_length, 
                              require_valid=require_valid_segs,
                              auto_partition=auto_partition_low_quality_segments)
                           
            except Exception as e:
                warn(str(e))
                newSc = None 
                if params.DEBUG:
                    raise
                else:
                    pass        
            if newSc:
                scenes.append(newSc)
            
        segs = []
        for sc in scenes:
            segs.extend(sc.segments)
#            for (segid, start, end) in sc:
#                print "segid, start, end:",segid, start, end
#                _, all_start, all_end = get_chunk(self.all_data, 0, start, end)
#                _, fix_start, fix_end = get_chunk(self.fix_data, 0, start, end)
#                new_seg = Segment(segid, self.all_data[all_start:all_end],
#                                  self.fix_data[fix_start:fix_end], aois=aoilist,
#                                  prune_length=prune_length)
#                new_seg.set_indices(all_start,all_end,fix_start,fix_end)
#                ret.append(new_seg)
        return segs , scenes
     
#    def process_dynamic(self, segfile = None, scenes = None,  aoifile = None, 
#                        aoilist = None , prune_length = None, media_offset = (0,0)):
#        """Processes the data for one recording (i.e, one complete experiment session)
#        
#                
#        Args:
#            segfile: If not None, a string containing the name of the segfile 
#                with segment definitions in following format:
#                <Scene_ID>\t<Segment_ID>\t<start time>\t<end time>\n
#                e.g.:
#                s1    seg1    0    5988013
#                With one segment definition per line
#            scenelist: If not None, a list of Scene objects
#            *Note: Both segfile and scenelist cannot be None
#                
#            aoifile: If not None, a string conatining the name of the aoifile 
#                with definitions of the "AOI"s.
#            aoilist: If not None, a list of "AOI"s.
#            *Note: if aoifile is not None, aoilist will be ignored
#             
#            prune_length: If not None, an integer that specifies the time 
#                interval (in ms) from the begining of each segment in which
#                samples are considered in calculations.  This can be used if, 
#                for example, you only wish to consider data in the first 
#                1000 ms of each segment. In this case (prune_length = 1000),
#                all data beyond the first 1000ms of the start of the "Segment"s
#                will be disregarded.
#            
#            require_valid_segs: a boolean flag determining whether invalid "Segment"s
#                will be ignored when calculating the features or not. default = True
#                 
#            auto_partition_low_quality_segments: a boolean flag determining whether
#                EMDAT should automatically split the "Segment"s which have low sample quality
#                into two new ssub "Segment"s discarding the largest invalid sample gap in 
#                the "Segment". default = False
#        Returns:
#            a list of Dynamic_Segment objects 
#        """   
#        if segfile != None:
#            scenes = read_segs(segfile)
#            print "Done reading the segments!"
#        elif scenes == None:
#            print "Error in scene file"
#            
#        ret = []
#        
#        if aoifile!= None:
#            aoilist = read_aois_Tobii(aoifile)
#            print "Done reading the AOIs!"
#        elif aoilist == None:
#            aoilist = []
#            print "No AOIs defined!"
#
##        all_ind = 0
##        fix_ind = 0
#        for sc in scenes.values():
#            for (segid, start, end) in sc:
#                print "segid, start, end:",segid, start, end
#                _, all_start, all_end = get_chunk(self.all_data, 0, start, end)
#                _, fix_start, fix_end = get_chunk(self.fix_data, 0, start, end)
#                ret.append(Dynamic_Segment(segid, self.all_data[all_start:all_end],
#                                           self.fix_data[fix_start:fix_end], aois=aoilist,
#                                           prune_length=prune_length))
#        return ret
           
            



def read_all_data(all_file):
    """Returns a list of "Datapoint"s read from an "All-Data" file.

    Args:
        all_file:A string containing the name of the 'All-Data.tsv' file output by the
            Tobii software.
    Returns:
        a list of "Datapoint"s
    """
    # with open(all_file, 'r') as f:
    #     lines = f.readlines()    
    
    alld = map(Datapoint, all_file)
   
    return list(filter(lambda x: x.is_None!=True,alld))


def read_fixation_data(fixation_file, media_offset = (0,0)):
    """Returns a list of "Fixation"s read from an "Fixation-Data" file. 

    Args:
        fixation_file: A string containing the name of the 'Fixation-Data.tsv' file output by the
            Tobii software.
        media_offset: the coordinates of the top left corner of the window
                showing the interface under study. (0,0) if the interface was
                in full screen (default value) 
    Returns:
        a list of "Fixation"s
    """

    

    return list(map(lambda x: Fixation(x, media_offset=media_offset), 
               fixation_file))

def read_aois_Tobii(aoifile):
    """Returns a list of "AOI"s read from a '.aoi' file.
    
    The '.aoi' files have pairs of lines of the form:
    aoiname[tab]point1x,point1y[tab]point2x,point2y[tab]...[new line]
    #[tab]start1,end1[tab]...[new line]
    
    The first line determines name of the AOI and the coordinates of each vertex of 
    the polygon that determines the boundaries of the AOI.
    The second line which starts with a '#' is optional and determines the time
    intervals when the AOI is active. If the second line does not exist the AOI will
    be active throughout the whole session (global AOI). 
    *Note: If the AOIs are exported from Tobii software the '.aoi' file will only have the 
    first line for each AOI and you need to override this method to generate AOIs that are
    active only at certain times (non-global AOI). 

    Args:
        aoifile: A string containing the name of the '.aoi' file
        
    Returns:
        a list of "AOI"s
    """
    # with open(aoifile, 'r') as f:
    #     aoilines = f.readlines()

    return read_aoilines(aoifile)

def read_aoilines(aoilines):
    """
    Args:
        aoilines: List of lines from a '.aoi' file

    Returns:
        list of AOIs
    """
    aoilist = []
    polyin = []
    last_aid = ''
    for aoi in aoilines:
        new_aoi = AOI(aoi.type_of_element,[(int(aoi.x),int(aoi.y)),(int(aoi.x)+int(aoi.width),int(aoi.y)),(int(aoi.x)+int(aoi.width),int(aoi.y)+int(aoi.height)),(int(aoi.x),int(aoi.y)+int(aoi.height))])
        aoilist.append(new_aoi)
    # for line in aoilines:
    #     chunks = line.strip().split('\t')
    #     print(chunks)
    #     if chunks[0].startswith('#'): # second line
    #         if polyin:
    #             seq = []
    #             for v in chunks[1:]:
    #                 seq.append((eval(v)))

    #             aoi = AOI(last_aid, polyin, [], seq)
    #             aoilist.append(aoi)
    #             polyin = []
    #         else:
    #             raise Exception('error in the AOI file')
    #     else:
    #         if polyin: # global AOI
    #             aoi = AOI(last_aid, polyin, [], [])
    #             aoilist.append(aoi)
    #             polyin = []

    #         last_aid = chunks[0] # first line
    #         for v in chunks[1:]:
    #             polyin.append((eval(v)))

    # if polyin: # last (global) AOI
    #     aoi = AOI(last_aid, polyin, [], [])
    #     aoilist.append(aoi)

    return aoilist


def read_segs(segfile):
    """Returns a dict with scid as the key and segments as value from a '.seg' file.
    
    A '.seg' file consists of a set of lines with the following format:
    scene_name[\t]segment_name[\t]start_time[\t]end_time[\n]
    
    scene_name is the id of the Scene that this Segment belongs to,
    segment_name is the id of the Segement,
    and start_time and end_time determines the time interval for the Segment

    Args:
        segfile: A string containing the name of the '.seg' file
        
    Returns:
        a dict with scid as the key and segments as value 
    """
    scenes = {}
    
    scenes["problem"] = [("Segment",float(segfile.startTimestamp),float(segfile.lastTimestamp))]
    # with open(segfile, 'r') as f:
    #     seglines = f.readlines() 
    
    # for l in seglines:
    #     l = l.strip()
    #     l = l.split('\t')
    #     if l[0] in scenes:
    #         scenes[l[0]].append((l[1], int(l[2]), int(l[3]) ))
    #     else:
    #         scenes[l[0]]=[(l[1], int(l[2]), int(l[3]) )]

    print(scenes)
    return scenes







f"""
UBC Eye Movement Data Analysys Toolkit
Created on 2012-08-23

@author: skardan
"""

from skgaze.EMDAT.data_structures import *
import skgaze.EMDAT.Recording
import skgaze.EMDAT.params as params
from skgaze.EMDAT.Participant import *
from skgaze.EMDAT.AOI import AOI
from skgaze.EMDAT.Scene import Scene
from skgaze.EMDAT.utils import *
from math import ceil, floor


class BasicParticipant(Participant):
    """
    This is a sample child class based on the Participant class that implements all the 
    placeholder methods in the Participant class for a basic project
    """
    def __init__(self, pid, eventfile, datafile, fixfile, segfile, log_time_offset = None, aoifile = None, prune_length= None, 
                 require_valid_segs = True, auto_partition_low_quality_segments = False):
        """Inits BasicParticipant class
        Args:
            pid: Participant id
            
            eventfile: a string containing the name of the "Event-Data.tsv" file for this participant
            
            datafile: a string containing the name of the "all-Data.tsv" file for this participant
            
            fixfile: a string containing the name of the "Fixation-Data.tsv" file for this participant
            
            segfile: a string containing the name of the '.seg' file for this participant
            
            log_time_offset: If not None, an integer indicating the time offset between the 
                external log file and eye tracking logs
            
            aoifile: If not None, a string conatining the name of the '.aoi' file 
                with definitions of the "AOI"s.
            
            prune_length: If not None, an integer that specifies the time 
                interval (in ms) from the begining of each Segment in which
                samples are considered in calculations.  This can be used if, 
                for example, you only wish to consider data in the first 
                1000 ms of each Segment. In this case (prune_length = 1000),
                all data beyond the first 1000ms of the start of the "Segment"s
                will be disregarded.
                
            auto_partition_low_quality_segments: a boolean indicating whether EMDAT should 
                split the "Segment"s which have low sample quality, into two new 
                sub "Segment"s discarding the largest gap of invalid samples.
            
        Yields:
            a BasicParticipant object
        """
        

        Participant.__init__(self, pid, eventfile, datafile, fixfile, segfile, log_time_offset, aoifile, prune_length, 
                 require_valid_segs, auto_partition_low_quality_segments)   #calling the Participan's constructor
        
        print("reading the files")
        self.features={}
        rec = Recording.Recording(datafile, datafile, params.MEDIA_OFFSET)
        print("Done!")
        
        scenelist,self.numofsegments = partition_Basic(segfile)
        print("partition done!")
        if aoifile != None:
            aois = Recording.read_aois_Tobii(aoifile)
        else:
            aois = None
        self.features['numofsegments']= self.numofsegments
        self.segments, self.scenes = rec.process_rec(scenelist = scenelist,aoilist = aois,prune_length = prune_length, require_valid_segs = require_valid_segs, 
                                                     auto_partition_low_quality_segments = auto_partition_low_quality_segments)
        self.whole_scene = Scene('P'+str(pid),[],rec.all_data,rec.fix_data, Segments = self.segments, aoilist = aois,prune_length = prune_length, require_valid = require_valid_segs )
        self.scenes.insert(0,self.whole_scene)

    
    def getAoiData(self, name):
        return self.whole_scene.aoi_data[name].features
       
        

               
def read_participants_Basic(datadir, user_list ,pids, prune_length = None, aoifile = None, log_time_offsets=None, 
                          require_valid_segs = True, auto_partition_low_quality_segments = False):
    
    participants = []
    if log_time_offsets == None:    #setting the default offset which is 1 sec
        log_time_offsets = [1]*len(pids) 
        
    for rec,pid,offset in zip(user_list,pids,log_time_offsets):
        print("pid:", pid)
        if rec<10:
            allfile = datadir+'/P0'+str(rec)+'-All-Data.tsv'
            fixfile = datadir+'/P0'+str(rec)+'-Fixation-Data.tsv'
            evefile = datadir+'/P0'+str(rec)+'-Event-Data.tsv'
            segfile = datadir+'/P0'+str(rec)+'.seg'
        else:
            allfile = datadir+'/P'+str(rec)+'-All-Data.tsv'
            fixfile = datadir+'/P'+str(rec)+'-Fixation-Data.tsv'
            evefile = datadir+'/P'+str(rec)+'-Event-Data.tsv'
            segfile = datadir+'/P'+str(rec)+'.seg'
        print(allfile)
        import os.path
        if os.path.exists(allfile):
            p = BasicParticipant(rec, evefile, allfile, fixfile, segfile, log_time_offset = offset, 
                                aoifile=aoifile, prune_length = prune_length, require_valid_segs = require_valid_segs,
                                auto_partition_low_quality_segments = auto_partition_low_quality_segments)
            participants.append(p)
        else:
            print("Error reading participant files for: "+pid)
    return participants

def partition_Basic(segfile):
    """Generates the scenelist based on a .seg file
    
    Args:
        segfile: a string containing the name of the '.seg' file
    
    Returns:
        a dict with scid as the key and tuples of (segid, start , end) for segments that belong to
            that scene as value
        an integer determining the number of segments
    """
    scenelist = Recording.read_segs(segfile)
    segcount = 0
    for l in scenelist.values():
        segcount += len(l)
    return scenelist, segcount

